@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2006-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -68,6 +68,7 @@ G_BEGIN_DECLS
 #define GPM_CONF_NOTIFY_ACADAPTER	GPM_CONF_DIR "/notify_ac_adapter"
 #define GPM_CONF_NOTIFY_BATTCHARGED	GPM_CONF_DIR "/notify_fully_charged"
 #define GPM_CONF_NOTIFY_HAL_ERROR	GPM_CONF_DIR "/notify_hal_error"
+#define GPM_CONF_NOTIFY_LOW_POWER	GPM_CONF_DIR "/notify_low_power"
 
 #define GPM_CONF_CAN_SUSPEND		GPM_CONF_DIR "/can_suspend"
 #define GPM_CONF_CAN_HIBERNATE		GPM_CONF_DIR "/can_hibernate"
@@ -118,8 +119,10 @@ G_BEGIN_DECLS
 #define GPM_CONF_BATTERY_CPUFREQ_POLICY	GPM_CONF_DIR "/cpufreq_battery_policy"
 #define GPM_CONF_BATTERY_CPUFREQ_VALUE	GPM_CONF_DIR "/cpufreq_battery_performance"
 #define GPM_CONF_USE_NICE		GPM_CONF_DIR "/cpufreq_consider_nice"
+#define GPM_CONF_SHOW_CPUFREQ_UI	GPM_CONF_DIR "/show_cpufreq_ui"
 
 #define GPM_CONF_STAT_SHOW_LEGEND	GPM_CONF_DIR "/statistics_show_legend"
+#define GPM_CONF_STAT_SHOW_AXIS_LABELS	GPM_CONF_DIR "/statistics_show_axis_labels"
 #define GPM_CONF_STAT_SHOW_EVENTS	GPM_CONF_DIR "/statistics_show_events"
 #define GPM_CONF_STAT_GRAPH_TYPE	GPM_CONF_DIR "/statistics_graph_type"
 
