@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2006-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -22,7 +22,8 @@
 #include "config.h"
 
 #include <glib.h>
-
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
 #include <glib/gi18n.h>
 #include <string.h>
 #include "gpm-inhibit.h"
@@ -41,12 +42,14 @@ typedef struct
 	gchar			*application;
 	gchar			*reason;
 	gchar			*connection;
+	gboolean		 manual;
 	guint32			 cookie;
 } GpmInhibitData;
 
 struct GpmInhibitPrivate
 {
-	GSList			*list;
+	GSList			*list_auto;
+	GSList			*list_manual;
 	GpmDbusMonitor		*dbus_monitor;
 	GpmConf			*conf;
 	gboolean		 ignore_inhibits;
@@ -89,9 +92,15 @@ gpm_inhibit_find_cookie (GpmInhibit *inhibit, guint32 cookie)
 {
 	GpmInhibitData *data;
 	GSList	       *ret;
-	ret = g_slist_find_custom (inhibit->priv->list, &cookie,
+	/* search auto list */
+	ret = g_slist_find_custom (inhibit->priv->list_auto, &cookie,
 				   gpm_inhibit_cookie_compare_func);
-	if (! ret) {
+	/* search manual list */
+	if (ret == NULL) {
+		ret = g_slist_find_custom (inhibit->priv->list_manual, &cookie,
+					   gpm_inhibit_cookie_compare_func);
+	}
+	if (ret == NULL) {
 		return NULL;
 	}
 	data = (GpmInhibitData *) ret->data;
@@ -119,7 +128,7 @@ gpm_inhibit_generate_cookie (GpmInhibit *inhibit)
 }
 
 /**
- * gpm_inhibit_add:
+ * gpm_inhibit_inhibit:
  * @connection: Connection name, e.g. ":0.13"
  * @application:	Application name, e.g. "Nautilus"
  * @reason: Reason for inhibiting, e.g. "Copying files"
@@ -131,35 +140,95 @@ gpm_inhibit_generate_cookie (GpmInhibit *inhibit)
  *
  * Return value: a new random cookie.
  **/
-guint32
-gpm_inhibit_add (GpmInhibit  *inhibit,
-		 const gchar *connection,
-		 const gchar *application,
-		 const gchar *reason)
+void
+gpm_inhibit_inhibit_auto (GpmInhibit  *inhibit,
+		          const gchar *application,
+		          const gchar *reason,
+		          DBusGMethodInvocation *context,
+		          GError     **error)
 {
-	GpmInhibitData *data = g_new (GpmInhibitData, 1);
+	const gchar *connection;
+	GpmInhibitData *data;
+
+	/* as we are async, we can get the sender */
+	connection = dbus_g_method_get_sender (context);
 
 	/* handle where the application does not add required data */
 	if (connection == NULL ||
 	    application == NULL ||
 	    reason == NULL) {
-		gpm_warning ("Recieved Inhibit, but application "
+		gpm_warning ("Recieved InhibitAuto, but application "
 			     "did not set the parameters correctly");
-		return -1;
+		dbus_g_method_return (context, -1);
+		return;
 	}
 
 	/* seems okay, add to list */
+	data = g_new (GpmInhibitData, 1);
 	data->cookie = gpm_inhibit_generate_cookie (inhibit);
 	data->application = g_strdup (application);
 	data->connection = g_strdup (connection);
 	data->reason = g_strdup (reason);
+	data->manual = FALSE;
 
-	inhibit->priv->list = g_slist_append (inhibit->priv->list,
-					      (gpointer) data);
+	inhibit->priv->list_auto = g_slist_append (inhibit->priv->list_auto, (gpointer) data);
 
-	gpm_debug ("Recieved Inhibit from '%s' (%s) because '%s' saving as #%i",
+	gpm_debug ("Recieved InhibitAuto from '%s' (%s) because '%s' saving as #%i",
 		   data->application, data->connection, data->reason, data->cookie);
-	return data->cookie;
+
+	dbus_g_method_return (context, data->cookie);
+}
+
+/**
+ * gpm_inhibit_inhibit_manual:
+ * @connection: Connection name, e.g. ":0.13"
+ * @application:	Application name, e.g. "Nautilus"
+ * @reason: Reason for inhibiting, e.g. "Copying files"
+ *
+ * Allocates a random cookie used to identify the connection, as multiple
+ * inhibit requests can come from one caller sharing a dbus connection.
+ * We need to refcount internally, and data is saved in the GpmInhibitData
+ * struct.
+ *
+ * Return value: a new random cookie.
+ **/
+void
+gpm_inhibit_inhibit_manual (GpmInhibit  *inhibit,
+		            const gchar *application,
+		            const gchar *reason,
+		            DBusGMethodInvocation *context,
+		            GError     **error)
+{
+	const gchar *connection;
+	GpmInhibitData *data;
+
+	/* as we are async, we can get the sender */
+	connection = dbus_g_method_get_sender (context);
+
+	/* handle where the application does not add required data */
+	if (connection == NULL ||
+	    application == NULL ||
+	    reason == NULL) {
+		gpm_warning ("Recieved InhibitManual, but application "
+			     "did not set the parameters correctly");
+		dbus_g_method_return (context, -1);
+		return;
+	}
+
+	/* seems okay, add to list */
+	data = g_new (GpmInhibitData, 1);
+	data->cookie = gpm_inhibit_generate_cookie (inhibit);
+	data->application = g_strdup (application);
+	data->connection = g_strdup (connection);
+	data->reason = g_strdup (reason);
+	data->manual = TRUE;
+
+	inhibit->priv->list_manual = g_slist_append (inhibit->priv->list_manual, (gpointer) data);
+
+	gpm_debug ("Recieved InhibitManual from '%s' (%s) because '%s' saving as #%i",
+		   data->application, data->connection, data->reason, data->cookie);
+
+	dbus_g_method_return (context, data->cookie);
 }
 
 /* free one element in GpmInhibitData struct */
@@ -173,32 +242,50 @@ gpm_inhibit_free_data_object (GpmInhibitData *data)
 }
 
 /**
- * gpm_inhibit_remove:
- * @connection: Connection name
+ * gpm_inhibit_un_inhibit:
  * @application:	Application name
  * @cookie: The cookie that we used to register
  *
  * Removes a cookie and associated data from the GpmInhibitData struct.
  **/
-void
-gpm_inhibit_remove (GpmInhibit  *inhibit,
-		    const gchar *connection,
-		    guint32	 cookie)
+gboolean
+gpm_inhibit_un_inhibit (GpmInhibit  *inhibit,
+		        guint32      cookie,
+		        GError     **error)
 {
 	GpmInhibitData *data;
 
 	/* Only remove the correct cookie */
 	data = gpm_inhibit_find_cookie (inhibit, cookie);
-	if (data) {
-		gpm_debug ("UnInhibit okay on '%s' as #%i",
-			   connection, cookie);
-		gpm_inhibit_free_data_object (data);
-		inhibit->priv->list = g_slist_remove (inhibit->priv->list,
-						      (gconstpointer) data);
-	} else {
+	if (data == NULL) {
 		gpm_warning ("Cannot find registered program for #%i, so "
 			     "cannot do UnInhibit", cookie);
+		return FALSE;
 	}
+	gpm_debug ("UnInhibit okay #%i", cookie);
+	gpm_inhibit_free_data_object (data);
+
+	/* remove it from the correct list */
+	if (data->manual == TRUE) {
+		inhibit->priv->list_manual = g_slist_remove (inhibit->priv->list_manual, (gconstpointer) data);
+	} else {
+		inhibit->priv->list_auto = g_slist_remove (inhibit->priv->list_auto, (gconstpointer) data);
+	}
+	return TRUE;
+}
+
+/**
+ * gpm_inhibit_get_requests:
+ *
+ * Gets a list of inhibits.
+ **/
+gboolean
+gpm_inhibit_get_requests (GpmInhibit *inhibit,
+			  gchar	   ***requests,
+			  GError    **error)
+{
+	gpm_warning ("Not implimented");
+	return FALSE;
 }
 
 /**
@@ -217,14 +304,18 @@ gpm_inhibit_remove_dbus (GpmInhibit  *inhibit,
 	int a;
 	GpmInhibitData *data;
 	/* Remove *any* connections that match the connection */
-	for (a=0; a<g_slist_length (inhibit->priv->list); a++) {
-		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, a);
+	for (a=0; a<g_slist_length (inhibit->priv->list_auto); a++) {
+		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list_auto, a);
 		if (strcmp (data->connection, connection) == 0) {
 			gpm_debug ("Auto-revoked idle inhibit on '%s'.",
 				   data->application);
 			gpm_inhibit_free_data_object (data);
-			inhibit->priv->list = g_slist_remove (inhibit->priv->list,
-							      (gconstpointer) data);
+			/* remove it from the correct list */
+			if (data->manual == TRUE) {
+				inhibit->priv->list_manual = g_slist_remove (inhibit->priv->list_manual, (gconstpointer) data);
+			} else {
+				inhibit->priv->list_auto = g_slist_remove (inhibit->priv->list_auto, (gconstpointer) data);
+			}
 		}
 	}
 	return;
@@ -253,26 +344,54 @@ dbus_noc_session_cb (GpmDbusMonitor *dbus_monitor,
 }
 
 /**
- * gpm_inhibit_check:
+ * gpm_inhibit_is_valid
  *
  * Checks to see if we are being stopped from performing an action.
  *
- * Return value: TRUE if the action is OK, i.e. we have *not* been inhibited
+ * TRUE if the action is OK, i.e. we have *not* been inhibited.
  **/
 gboolean
-gpm_inhibit_check (GpmInhibit *inhibit)
+gpm_inhibit_is_valid (GpmInhibit *inhibit,
+		      gboolean	  user_action,
+		      gboolean   *valid,
+		      GError    **error)
 {
-	if (inhibit->priv->ignore_inhibits) {
+	guint length_auto;
+	guint length_manual;
+
+	length_auto = g_slist_length (inhibit->priv->list_auto);
+	length_manual = g_slist_length (inhibit->priv->list_manual);
+
+	if (inhibit->priv->ignore_inhibits == TRUE) {
 		gpm_debug ("Inhibit ignored through gconf policy!");
-		return TRUE;
+		*valid = TRUE;
 	}
 
-	if (g_slist_length (inhibit->priv->list) == 0) {
-		return TRUE;
+	if (user_action == TRUE) {
+		/* if user clicks button or closes lid then only manual inhibitors
+		   can stop the action. */
+		if (length_manual == 0) {
+			gpm_debug ("Valid as no manual inhibitors");
+			*valid = TRUE;
+		} else {
+			/* we have at least one application blocking the action */
+			gpm_debug ("We have %i valid manual inhibitors", length_manual);
+			*valid = FALSE;
+		}
+	} else {
+		/* A manual or auto inhibit can stop the action if not consciously done. */
+		if (length_auto == 0 && length_manual == 0) {
+			gpm_debug ("Valid as no auto or manual inhibitors");
+			*valid = TRUE;
+		} else {
+			/* we have at least one application blocking the action */
+			gpm_debug ("We have %i valid auto and %i manual inhibitors", length_auto, length_manual);
+			*valid = FALSE;
+		}
 	}
 
-	/* we have at least one application blocking the action */
-	return FALSE;
+	/* we always return successful for DBUS */
+	return TRUE;
 }
 
 /**
@@ -293,8 +412,9 @@ gpm_inhibit_get_message (GpmInhibit  *inhibit,
 	guint a;
 	GpmInhibitData *data;
 
-	if (g_slist_length (inhibit->priv->list) == 1) {
-		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, 0);
+	/* todo: use manual also */
+	if (g_slist_length (inhibit->priv->list_auto) == 1) {
+		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list_auto, 0);
 		g_string_append_printf (message, "<b>%s</b> ",
 					data->application);
 		g_string_append_printf (message, _("has stopped the %s from "
@@ -305,8 +425,8 @@ gpm_inhibit_get_message (GpmInhibit  *inhibit,
 	} else {
 		g_string_append_printf (message, _("Multiple applications have stopped the "
 					"%s from taking place."), action);
-		for (a=0; a<g_slist_length (inhibit->priv->list); a++) {
-			data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, a);
+		for (a=0; a<g_slist_length (inhibit->priv->list_auto); a++) {
+			data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list_auto, a);
 			g_string_append_printf (message,
 						"\n<b>%s</b> : <i>%s</i>",
 						data->application, data->reason);
@@ -344,7 +464,8 @@ static void
 gpm_inhibit_init (GpmInhibit *inhibit)
 {
 	inhibit->priv = GPM_INHIBIT_GET_PRIVATE (inhibit);
-	inhibit->priv->list = NULL;
+	inhibit->priv->list_auto = NULL;
+	inhibit->priv->list_manual = NULL;
 	inhibit->priv->dbus_monitor = gpm_dbus_monitor_new ();
 	g_signal_connect (inhibit->priv->dbus_monitor, "noc-session",
 			  G_CALLBACK (dbus_noc_session_cb), inhibit);
@@ -372,11 +493,16 @@ gpm_inhibit_finalize (GObject *object)
 	inhibit->priv = GPM_INHIBIT_GET_PRIVATE (inhibit);
 
 	/* remove items in list and free */
-	for (a=0; a<g_slist_length (inhibit->priv->list); a++) {
-		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, a);
+	for (a=0; a<g_slist_length (inhibit->priv->list_manual); a++) {
+		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list_manual, a);
 		gpm_inhibit_free_data_object (data);
 	}
-	g_slist_free (inhibit->priv->list);
+	for (a=0; a<g_slist_length (inhibit->priv->list_auto); a++) {
+		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list_auto, a);
+		gpm_inhibit_free_data_object (data);
+	}
+	g_slist_free (inhibit->priv->list_manual);
+	g_slist_free (inhibit->priv->list_auto);
 
 	g_object_unref (inhibit->priv->conf);
 	g_object_unref (inhibit->priv->dbus_monitor);
