@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -23,7 +23,7 @@
 #define __GPMINHIBIT_H
 
 #include <glib-object.h>
-#include "gpm-power.h"
+#include <dbus/dbus-glib.h>
 
 G_BEGIN_DECLS
 
@@ -49,20 +49,32 @@ typedef struct
 
 GpmInhibit	*gpm_inhibit_new			(void);
 GType		 gpm_inhibit_get_type			(void);
-
-void		 gpm_inhibit_set_power			(GpmInhibit	*inhibit,
-							 GpmPower	*power);
-guint32		 gpm_inhibit_add			(GpmInhibit	*inhibit,
-							 const gchar	*connection,
-							 const gchar	*application,
-							 const gchar	*reason);
-void		 gpm_inhibit_remove			(GpmInhibit	*inhibit,
-							 const gchar	*connection,
-							 guint32	 cookie);
-gboolean	 gpm_inhibit_check			(GpmInhibit	*inhibit);
+gboolean	 gpm_inhibit_is_valid			(GpmInhibit	*inhibit,
+							 gboolean	 user_action,
+							 gboolean	*valid,
+							 GError		**error);
 void		 gpm_inhibit_get_message		(GpmInhibit	*inhibit,
 							 GString	*message,
 							 const gchar	*action);
+
+void		 gpm_inhibit_inhibit_manual		(GpmInhibit	*inhibit,
+							 const gchar	*application,
+							 const gchar	*reason,
+							 DBusGMethodInvocation *context,
+							 GError		**error);
+void		 gpm_inhibit_inhibit_auto		(GpmInhibit	*inhibit,
+							 const gchar	*application,
+							 const gchar	*reason,
+							 DBusGMethodInvocation *context,
+							 GError		**error);
+gboolean	 gpm_inhibit_un_inhibit			(GpmInhibit	*inhibit,
+							 guint32	 cookie,
+							 GError		**error);
+gboolean	 gpm_inhibit_get_requests		(GpmInhibit	*inhibit,
+							 gchar		***requests,
+							 GError		**error);
+
+
 G_END_DECLS
 
 #endif	/* __GPMINHIBIT_H */
