@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2005-2006 Jaap Haitsma <jaap@haitsma.org>
  *
  * Licensed under the GNU General Public License Version 2
@@ -47,7 +47,8 @@ typedef struct {
 	guint		design_charge;
 	guint		last_full_charge;
 	guint		current_charge;
-	guint		charge_rate_smoothed;	/* exp ave smoothed */
+	guint		charge_rate_smoothed;	/* exp ave smoothed... calculated for caches */
+	guint		charge_rate_previous;	/* used to calculate the exp ave */
 	guint		charge_rate_raw;	/* no smoothing done */
 	guint		percentage_charge;
 	guint		remaining_time;
@@ -66,7 +67,6 @@ typedef struct {
 	gchar		*technology;
 	gchar		*serial;
 	gchar		*model;
-	guint		 charge_rate_previous;
 	GpmPowerKind	 battery_kind;
 	GpmPowerStatus	 battery_status;
 	GpmPowerUnit	 unit;
@@ -90,15 +90,8 @@ typedef struct
 typedef struct
 {
 	GObjectClass parent_class;
-	void		(* button_pressed)		(GpmPower	*power,
-							 const gchar	*type,
-							 gboolean	 state);
-	void		(* ac_state_changed)		(GpmPower	*power,
-							 gboolean	 on_ac);
 	void		(* battery_status_changed)	(GpmPower	*power,
 							 GpmPowerKind	 battery_kind);
-	void		(* battery_removed)		(GpmPower	*power,
-							 const gchar	*udi);
 	void		(* battery_perhaps_recall)	(GpmPower	*power,
 							 const gchar	*oem_vendor,
 							 const gchar	*website);
@@ -106,9 +99,6 @@ typedef struct
 
 GType		 gpm_power_get_type			(void);
 GpmPower	*gpm_power_new				(void);
-gboolean	 gpm_power_get_on_ac			(GpmPower	*power,
-							 gboolean	*on_ac,
-							 GError		**error);
 gboolean	 gpm_power_get_battery_status		(GpmPower	*power,
 							 GpmPowerKind	 battery_kind,
 							 GpmPowerStatus *battery_status);
