@@ -580,7 +580,8 @@ gpm_statistics_type_combo_changed_cb (GtkWidget      *widget,
 	} else if (strcmp (value, ACTION_VOLTAGE_TEXT) == 0) {
 		type = ACTION_VOLTAGE;
 	} else {
-		g_assert (FALSE);
+		gpm_debug ("type not known, ignoring");
+		return;
 	}
 	g_free (value);
 
@@ -657,6 +658,12 @@ static gboolean
 gpm_statistics_graph_refresh (gpointer data)
 {
 	GpmStatistics *statistics = GPM_STATISTICS (data);
+
+	if (gpm_proxy_is_connected (statistics->priv->gproxy) == FALSE) {
+		gpm_warning ("not connected!");
+		return FALSE;
+	}
+
 	gpm_debug ("refreshing graph type '%s'", statistics->priv->graph_type);
 	gpm_statistics_refresh_data (statistics);
 	gpm_statistics_refresh_events (statistics);
