@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -45,8 +45,12 @@ typedef struct
 typedef struct
 {
 	GObjectClass	parent_class;
-	void	(* suspend)			(GpmTrayIcon *tray_icon);
-	void	(* hibernate)			(GpmTrayIcon *tray_icon);
+	void	(* suspend)				(GpmTrayIcon	*tray_icon);
+	void	(* hibernate)				(GpmTrayIcon	*tray_icon);
+	void	(* description_changed)			(GpmTrayIcon	*tray_icon,
+							 const gchar	**description);
+	void	(* icon_changed)			(GpmTrayIcon	*tray_icon,
+							 const gchar	**icon);
 } GpmTrayIconClass;
 
 GType		 gpm_tray_icon_get_type			(void);
@@ -58,6 +62,15 @@ void		 gpm_tray_icon_set_image_from_stock	(GpmTrayIcon	*icon,
 							 const gchar	*stock_id);
 void		 gpm_tray_icon_sync			(GpmTrayIcon	*icon);
 
+gboolean
+gpm_ui_get_description (GpmTrayIcon  *icon,
+		        gchar **mode_str,
+		        GError      **error);
+gboolean
+gpm_ui_get_icon (GpmTrayIcon  *icon,
+		 gchar **mode_str,
+		 GError      **error);
+
 G_END_DECLS
 
 #endif /* __GPM_TRAY_ICON_H */
